# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: graphsignal/profilers/tensorflow_proto/input_pipeline.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from graphsignal.profilers.tensorflow_proto import diagnostics_pb2 as graphsignal_dot_profilers_dot_tensorflow__proto_dot_diagnostics__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;graphsignal/profilers/tensorflow_proto/input_pipeline.proto\x12\x13tensorflow.profiler\x1a\x19google/protobuf/any.proto\x1a\x38graphsignal/profilers/tensorflow_proto/diagnostics.proto\"\x80\x03\n\x12\x42ottleneckAnalysis\x12\x15\n\rinput_percent\x18\x07 \x01(\x01\x12\x16\n\x0eoutput_percent\x18\x08 \x01(\x01\x12\x14\n\x0cidle_percent\x18\t \x01(\x01\x12\x17\n\x0f\x63ompute_percent\x18\n \x01(\x01\x12\x1c\n\x14input_classification\x18\x01 \x01(\t\x12\x17\n\x0finput_statement\x18\x02 \x01(\t\x12$\n\x1ckernel_launch_classification\x18\x03 \x01(\t\x12\x1f\n\x17kernel_launch_statement\x18\x04 \x01(\t\x12 \n\x18\x61ll_other_classification\x18\x05 \x01(\t\x12\x1b\n\x13\x61ll_other_statement\x18\x06 \x01(\t\x12)\n!device_collectives_classification\x18\x0b \x01(\t\x12$\n\x1c\x64\x65vice_collectives_statement\x18\x0c \x01(\t\"\\\n\x0bStepSummary\x12\x0f\n\x07\x61verage\x18\x01 \x01(\x01\x12\x1a\n\x12standard_deviation\x18\x02 \x01(\x01\x12\x0f\n\x07minimum\x18\x03 \x01(\x01\x12\x0f\n\x07maximum\x18\x04 \x01(\x01\"\xe0\x02\n\x15PerGenericStepDetails\x12\x13\n\x0bstep_number\x18\x01 \x01(\x05\x12\x11\n\tstep_name\x18\x0e \x01(\t\x12\x14\n\x0cstep_time_ms\x18\x02 \x01(\x01\x12\x17\n\x0funknown_time_ms\x18\x03 \x01(\x01\x12\x1a\n\x12host_wait_input_ms\x18\x0b \x01(\x01\x12\x19\n\x11host_to_device_ms\x18\x0c \x01(\x01\x12\x11\n\toutput_ms\x18\x05 \x01(\x01\x12\x19\n\x11\x64\x65vice_compute_ms\x18\x06 \x01(\x01\x12\x1b\n\x13\x64\x65vice_to_device_ms\x18\x07 \x01(\x01\x12\x1d\n\x15\x64\x65vice_collectives_ms\x18\r \x01(\x01\x12\x17\n\x0fhost_compute_ms\x18\x08 \x01(\x01\x12\x17\n\x0fhost_prepare_ms\x18\t \x01(\x01\x12\x17\n\x0fhost_compile_ms\x18\n \x01(\x01J\x04\x08\x04\x10\x05\"\xa5\x01\n\x12InputTimeBreakdown\x12\x1d\n\x15\x64\x65manded_file_read_us\x18\x01 \x01(\x01\x12\x1d\n\x15\x61\x64vanced_file_read_us\x18\x02 \x01(\x01\x12\x18\n\x10preprocessing_us\x18\x03 \x01(\x01\x12\x12\n\nenqueue_us\x18\x04 \x01(\x01\x12#\n\x1bunclassified_non_enqueue_us\x18\x05 \x01(\x01\"\xa6\x01\n\x0eInputOpDetails\x12\x0f\n\x07op_name\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\x04\x12\x12\n\ntime_in_ms\x18\x03 \x01(\x01\x12\x17\n\x0ftime_in_percent\x18\x04 \x01(\x01\x12\x17\n\x0fself_time_in_ms\x18\x05 \x01(\x01\x12\x1c\n\x14self_time_in_percent\x18\x06 \x01(\x01\x12\x10\n\x08\x63\x61tegory\x18\x07 \x01(\t\"\x84\x01\n#InputPipelineAnalysisRecommendation\x12\x0f\n\x07\x64\x65tails\x18\x01 \x03(\t\x12\x31\n\x13\x62ottleneck_analysis\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x19\n\x11summary_next_step\x18\x03 \x01(\t\"\x85\x06\n\x18GenericStepTimeBreakdown\x12\x41\n\x17unknown_time_ms_summary\x18\x01 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x44\n\x1ahost_wait_input_ms_summary\x18\t \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x43\n\x19host_to_device_ms_summary\x18\n \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12:\n\x10input_ms_summary\x18\x0b \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12;\n\x11output_ms_summary\x18\x03 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x43\n\x19\x64\x65vice_compute_ms_summary\x18\x04 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x45\n\x1b\x64\x65vice_to_device_ms_summary\x18\x05 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12G\n\x1d\x64\x65vice_collectives_ms_summary\x18\x0c \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x41\n\x17host_compute_ms_summary\x18\x06 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x41\n\x17host_prepare_ms_summary\x18\x07 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x41\n\x17host_compile_ms_summary\x18\x08 \x01(\x0b\x32 .tensorflow.profiler.StepSummaryJ\x04\x08\x02\x10\x03\"\x8a\x05\n\x1bInputPipelineAnalysisResult\x12\x15\n\rhardware_type\x18\t \x01(\t\x12;\n\x11step_time_summary\x18\x02 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12?\n\x15input_percent_summary\x18\x03 \x01(\x0b\x32 .tensorflow.profiler.StepSummary\x12\x15\n\rinput_percent\x18\x0b \x01(\x01\x12\x16\n\x0eoutput_percent\x18\r \x01(\x01\x12\x14\n\x0cidle_percent\x18\x0e \x01(\x01\x12\x17\n\x0f\x63ompute_percent\x18\x0f \x01(\x01\x12*\n\x0cstep_details\x18\x04 \x03(\x0b\x32\x14.google.protobuf.Any\x12\x45\n\x14input_time_breakdown\x18\x05 \x01(\x0b\x32\'.tensorflow.profiler.InputTimeBreakdown\x12=\n\x10input_op_details\x18\x06 \x03(\x0b\x32#.tensorflow.profiler.InputOpDetails\x12P\n\x0erecommendation\x18\x07 \x01(\x0b\x32\x38.tensorflow.profiler.InputPipelineAnalysisRecommendation\x12\x31\n\x13step_time_breakdown\x18\x08 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x35\n\x0b\x64iagnostics\x18\x0c \x01(\x0b\x32 .tensorflow.profiler.DiagnosticsJ\x04\x08\x01\x10\x02J\x04\x08\n\x10\x0b\x62\x06proto3')



_BOTTLENECKANALYSIS = DESCRIPTOR.message_types_by_name['BottleneckAnalysis']
_STEPSUMMARY = DESCRIPTOR.message_types_by_name['StepSummary']
_PERGENERICSTEPDETAILS = DESCRIPTOR.message_types_by_name['PerGenericStepDetails']
_INPUTTIMEBREAKDOWN = DESCRIPTOR.message_types_by_name['InputTimeBreakdown']
_INPUTOPDETAILS = DESCRIPTOR.message_types_by_name['InputOpDetails']
_INPUTPIPELINEANALYSISRECOMMENDATION = DESCRIPTOR.message_types_by_name['InputPipelineAnalysisRecommendation']
_GENERICSTEPTIMEBREAKDOWN = DESCRIPTOR.message_types_by_name['GenericStepTimeBreakdown']
_INPUTPIPELINEANALYSISRESULT = DESCRIPTOR.message_types_by_name['InputPipelineAnalysisResult']
BottleneckAnalysis = _reflection.GeneratedProtocolMessageType('BottleneckAnalysis', (_message.Message,), {
  'DESCRIPTOR' : _BOTTLENECKANALYSIS,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.BottleneckAnalysis)
  })
_sym_db.RegisterMessage(BottleneckAnalysis)

StepSummary = _reflection.GeneratedProtocolMessageType('StepSummary', (_message.Message,), {
  'DESCRIPTOR' : _STEPSUMMARY,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.StepSummary)
  })
_sym_db.RegisterMessage(StepSummary)

PerGenericStepDetails = _reflection.GeneratedProtocolMessageType('PerGenericStepDetails', (_message.Message,), {
  'DESCRIPTOR' : _PERGENERICSTEPDETAILS,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.PerGenericStepDetails)
  })
_sym_db.RegisterMessage(PerGenericStepDetails)

InputTimeBreakdown = _reflection.GeneratedProtocolMessageType('InputTimeBreakdown', (_message.Message,), {
  'DESCRIPTOR' : _INPUTTIMEBREAKDOWN,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.InputTimeBreakdown)
  })
_sym_db.RegisterMessage(InputTimeBreakdown)

InputOpDetails = _reflection.GeneratedProtocolMessageType('InputOpDetails', (_message.Message,), {
  'DESCRIPTOR' : _INPUTOPDETAILS,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.InputOpDetails)
  })
_sym_db.RegisterMessage(InputOpDetails)

InputPipelineAnalysisRecommendation = _reflection.GeneratedProtocolMessageType('InputPipelineAnalysisRecommendation', (_message.Message,), {
  'DESCRIPTOR' : _INPUTPIPELINEANALYSISRECOMMENDATION,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.InputPipelineAnalysisRecommendation)
  })
_sym_db.RegisterMessage(InputPipelineAnalysisRecommendation)

GenericStepTimeBreakdown = _reflection.GeneratedProtocolMessageType('GenericStepTimeBreakdown', (_message.Message,), {
  'DESCRIPTOR' : _GENERICSTEPTIMEBREAKDOWN,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.GenericStepTimeBreakdown)
  })
_sym_db.RegisterMessage(GenericStepTimeBreakdown)

InputPipelineAnalysisResult = _reflection.GeneratedProtocolMessageType('InputPipelineAnalysisResult', (_message.Message,), {
  'DESCRIPTOR' : _INPUTPIPELINEANALYSISRESULT,
  '__module__' : 'graphsignal.profilers.tensorflow_proto.input_pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.profiler.InputPipelineAnalysisResult)
  })
_sym_db.RegisterMessage(InputPipelineAnalysisResult)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BOTTLENECKANALYSIS._serialized_start=170
  _BOTTLENECKANALYSIS._serialized_end=554
  _STEPSUMMARY._serialized_start=556
  _STEPSUMMARY._serialized_end=648
  _PERGENERICSTEPDETAILS._serialized_start=651
  _PERGENERICSTEPDETAILS._serialized_end=1003
  _INPUTTIMEBREAKDOWN._serialized_start=1006
  _INPUTTIMEBREAKDOWN._serialized_end=1171
  _INPUTOPDETAILS._serialized_start=1174
  _INPUTOPDETAILS._serialized_end=1340
  _INPUTPIPELINEANALYSISRECOMMENDATION._serialized_start=1343
  _INPUTPIPELINEANALYSISRECOMMENDATION._serialized_end=1475
  _GENERICSTEPTIMEBREAKDOWN._serialized_start=1478
  _GENERICSTEPTIMEBREAKDOWN._serialized_end=2251
  _INPUTPIPELINEANALYSISRESULT._serialized_start=2254
  _INPUTPIPELINEANALYSISRESULT._serialized_end=2904
# @@protoc_insertion_point(module_scope)
